`timescale 1ns / 10ps

module gtx_frontend(gtx_rxn, gtx_rxp, gtx_txn, gtx_txp, gtx_refclk_n,
  gtx_refclk_p, pipe_rx_clk, pipe_clk, pll_locked, frontend_rst, refclk_locked,
  mgt_en, lfps_en, mgt_powerdown, rx_dontmess, rx_phy_ready, rx_resync,
  rx_detect_revpolarity, rx_detect_revpolarity_clear, rx_align_enable,
  rx_elecidle, receiver_detect, receiver_present, receiver_present_valid,
  pipe_rx, pipe_rx_k, pipe_rx_valid, pipe_tx, pipe_tx_k, pipe_tx_ready,
  version_gtx_1_0);

  input  gtx_rxn;
  input  gtx_rxp;
  input  gtx_refclk_n;
  input  gtx_refclk_p;
  input  mgt_en;
  input  lfps_en;
  input  mgt_powerdown;
  input  rx_dontmess;
  input  rx_resync;
  input  rx_detect_revpolarity;
  input  rx_align_enable;
  input  receiver_detect;
  input [31:0] pipe_tx;
  input [3:0] pipe_tx_k;
  output  gtx_txn;
  output  gtx_txp;
  output  pipe_rx_clk;
  output  pipe_clk;
  output  pll_locked;
  output  frontend_rst;
  output  refclk_locked;
  output  rx_phy_ready;
  output  rx_detect_revpolarity_clear;
  output  rx_elecidle;
  output  receiver_present;
  output  receiver_present_valid;
  output [31:0] pipe_rx;
  output [3:0] pipe_rx_k;
  output  pipe_rx_valid;
  output  pipe_tx_ready;
  output  version_gtx_1_0;
  reg  pll_locked;
  reg  frontend_rst;
  reg  rx_phy_ready;
  reg  rx_detect_revpolarity_clear;
  reg  rx_elecidle;
  reg  receiver_present;
  reg  receiver_present_valid;
  reg [2:0] mgt_reset_rx;
  reg [2:0] mgt_reset_tx;
  reg  txelecidle;
  reg  refclk_locked_d;
  reg  refclk_locked_sync;
  reg  cpll_lock_d;
  reg  cpll_lock_d2;
  reg  cpll_reset;
  reg  rxelecidle_d;
  reg  rxelecidle_d2;
  reg [4:0] elecidle_cnt;
  reg [31:0] txdata;
  reg [3:0] txdata_k;
  reg  txdata_extend;
  reg [1:0] lfps_toggler;
  reg  tx_coding_bypass;
  reg [6:0] wakeup_counter;
  reg  rxdlysreset;
  reg  rxresetdone_d;
  reg  rxresetdone_sync;
  reg  rxdlysresetdone_d;
  reg  rxdlysresetdone_sync;
  reg  rxphaligndone_d;
  reg  rxphaligndone_d2;
  reg  rxphaligndone_d3;
  reg [1:0] done_count;
  reg [2:0] ph_state;
  reg [1:0] powerdown;
  reg  mgt_powerdown_d;
  reg  txdetectrx;
  reg [6:0] polarity_debounce_cnt;
  reg  rxpolarity;
  reg  rx_align_enable_d;
  reg  rx_align_enable_d2;
  reg [16:0] rx_selfreset_cnt;
  reg  rx_selfreset;
  wire  reset_rx;
  wire  reset_tx;
  wire  gt0_rxelecidle_i;
  wire  gt0_txoutclk_i;
  wire  gt0_rxoutclk_i;
  wire  q2_clk0_refclk_i;
  wire  usrclk;
  wire  cpll_lock;
  wire  rxphaligndone;
  wire  rxdlysresetdone;
  wire  phystatus;
  wire [2:0] rxstatus;
  wire  txresetdone;
  wire  rxresetdone;

   parameter RXELECIDLE_DEBOUNCE = 14; // ~ f_usrclk * 0.1 us

   parameter refclk_freq = 125; // In MHz

   localparam st_idle = 0,
     st_rstdone = 1,
     st_start = 2,
     st_wait1 = 3,
     st_wait2 = 4;

   initial wakeup_counter = 0;

   assign version_gtx_1_0 = 1;

   assign pipe_tx_ready = 1;
   assign pipe_rx_valid = 1;
   assign refclk_locked = 1;

   assign reset_rx = mgt_reset_rx[2];
   assign reset_tx = mgt_reset_tx[2];

   assign pipe_clk = usrclk;

   always @(posedge usrclk)
     begin
	rx_phy_ready <= (ph_state == st_idle);

	// Polarity detection debouncing is required to clear the pipeline
	// of data before the next verdict.

	rx_detect_revpolarity_clear <= (polarity_debounce_cnt != 0);

	if (polarity_debounce_cnt != 0)
	  polarity_debounce_cnt <= polarity_debounce_cnt - 1;
	else if (rx_detect_revpolarity)
	  begin
	     polarity_debounce_cnt <= polarity_debounce_cnt - 1; // Wrap to max
	     rxpolarity <= !rxpolarity;
	  end

	// CPLL lock indicator is asynchronous
	cpll_lock_d <= cpll_lock;
	cpll_lock_d2 <= cpll_lock_d;
	pll_locked <= cpll_lock_d2;

	// gt0_rxelecidle_i is an async signal, hence metastability guarded
	rxelecidle_d <= gt0_rxelecidle_i;
	rxelecidle_d2 <= rxelecidle_d;

	if (!rxelecidle_d2)
	  elecidle_cnt <= RXELECIDLE_DEBOUNCE;
	else if (elecidle_cnt != 0)
	  elecidle_cnt <= elecidle_cnt - 1;

	rx_elecidle <= (elecidle_cnt == 0);

	if ((wakeup_counter == 125) || (cpll_lock_d2 && !pll_locked))
	  begin
	     mgt_reset_rx <= ~0;
	     mgt_reset_tx <= ~0;
	  end
	else
	  begin
	     mgt_reset_rx <= { mgt_reset_rx, 1'b0 };
	     mgt_reset_tx <= { mgt_reset_tx, 1'b0 };
	  end

	// Self-reset the rx part every 1 ms, unless the LTSSM is in a
	// mode it expects the MGT to work in, as indicated by rx_dontmess.
	// As the LTSSM asserts rx_resync before starting to rely on the
	// MGT, it's harmless if rx_selfreset is asserted slightly after
	// rx_dontmess goes high. On the other hand, this self reset
	// mechanism is pointless in all situations because of rx_resync,
	// except when the Polling.LFPS signal can't be detected, in which case
	// the bringup relies on detection of MGT data. In that case, the MGT's
	// receiver must be ensured not be get messed up by noise on the wire.
	// In short: This self reset thing is quite far fetched, and can be
	// removed if it causes any trouble.

	if (rx_selfreset_cnt == 124999)
	  begin
	     rx_selfreset <= !rx_dontmess;
	     rx_selfreset_cnt <= 0;
	  end
	else
	  begin
	     rx_selfreset <= 0;
	     rx_selfreset_cnt <= rx_selfreset_cnt + 1;
	  end

	if (rx_resync || rx_selfreset)
	  mgt_reset_rx <= ~0;

	refclk_locked_d <= refclk_locked;
	refclk_locked_sync <= refclk_locked_d;

	if (!refclk_locked_sync)
	  wakeup_counter <= 0;
	else if (!(&wakeup_counter))
	  wakeup_counter <= wakeup_counter + 1;

	cpll_reset <= (wakeup_counter == 15); // 15 was picked arbitrarily

	frontend_rst <= !(&wakeup_counter) || !pll_locked;

	lfps_toggler <= lfps_toggler + 1;

	powerdown <= mgt_powerdown ? 2'b10 : 2'b00;
	mgt_powerdown_d <= mgt_powerdown;

	if (mgt_powerdown_d && !mgt_powerdown)
	  mgt_reset_tx <= ~0;

	if (mgt_en)
	  begin
	     txelecidle <= 0;
	     txdata <= pipe_tx;
	     txdata_k <= pipe_tx_k;
	     txdata_extend <= 0;
	     tx_coding_bypass <= 0;
	  end
	else if (lfps_en)
	  begin
	     txelecidle <= 0;
	     txdata <= { 32{lfps_toggler[1]} };
	     txdata_extend <= lfps_toggler[1];
	     tx_coding_bypass <= 1;
	  end
	else // Electrical Idle
	  begin
	     txelecidle <= 1;
	  end

	receiver_present_valid <= txdetectrx && phystatus;
	receiver_present <= rxstatus[0];

	// txdetectrx is deasserted on the clock following phystatus assertion,
	// following the PIPE spec, and not one clock later, as shown in e.g.
	// UG476. It can't hurt deasserting it when the result is available,
	// and UG476's timing diagram isn't PIPE compliant.

	if (reset_tx || phystatus)
	  txdetectrx <= 0;
	else if (receiver_detect)
	  txdetectrx <= 1;

	// State machine for phase alignment.
	// NOTE: The state machine will not detect the completion of
	//       the phase alignment on GTP/GTH!

	rxresetdone_d <= rxresetdone;
	rxresetdone_sync <= rxresetdone_d;

	rxdlysresetdone_d <= rxdlysresetdone;
	rxdlysresetdone_sync <= rxdlysresetdone_d;

	rxphaligndone_d <= rxphaligndone;
	rxphaligndone_d2 <= rxphaligndone_d;
	rxphaligndone_d3 <= rxphaligndone_d2;

	if (rxphaligndone_d2 && !rxphaligndone_d3 && !(&done_count))
	  done_count <= done_count + 1; // Possibly overridden below

	rxdlysreset <= 0; // Possibly overridden below

	case (ph_state)
	  st_rstdone:
	    if (rxresetdone_sync)
	      ph_state <= st_start;

	  st_start:
	    begin
	       rxdlysreset <= 1;
	       done_count <= 0;
	       ph_state <= st_wait1;
	    end

	  st_wait1:
	    if (rxdlysresetdone_sync)
	      ph_state <= st_wait2;

	  st_wait2:
	    if (done_count > 1)
	      ph_state <= st_idle;

	  default:
	    ph_state <= st_idle; // st_idle itself is covered by default.
	endcase

	if (reset_rx)
	  ph_state <= st_rstdone;
     end

   always @(posedge pipe_rx_clk)
     begin
	rx_align_enable_d <= rx_align_enable;
	rx_align_enable_d2 <= rx_align_enable_d;

     end

   GTXE2_CHANNEL #
     (
      .SIM_RECEIVER_DETECT_PASS("TRUE"),
      .SIM_TX_EIDLE_DRIVE_LEVEL("X"),
      .SIM_RESET_SPEEDUP("FALSE"),
      .SIM_CPLLREFCLK_SEL(3'h1),
      .SIM_VERSION("4.0"),

      .ALIGN_COMMA_DOUBLE("FALSE"),
      .ALIGN_COMMA_ENABLE(10'h3ff),
      .ALIGN_COMMA_WORD(1),
      .ALIGN_MCOMMA_DET("TRUE"),
      .ALIGN_MCOMMA_VALUE(10'h283),
      .ALIGN_PCOMMA_DET("TRUE"),
      .ALIGN_PCOMMA_VALUE(10'h17c),
      .SHOW_REALIGN_COMMA("TRUE"),
      .RXSLIDE_AUTO_WAIT(7),
      .RXSLIDE_MODE("PCS"),
      .RX_SIG_VALID_DLY(10),

      .RX_DISPERR_SEQ_MATCH("TRUE"),
      .DEC_MCOMMA_DETECT("TRUE"),
      .DEC_PCOMMA_DETECT("TRUE"),
      .DEC_VALID_COMMA_ONLY("FALSE"),

      .CBCC_DATA_SOURCE_SEL("DECODED"),
      .CLK_COR_SEQ_2_USE("FALSE"),
      .CLK_COR_KEEP_IDLE("FALSE"),
      .CLK_COR_MAX_LAT(19),
      .CLK_COR_MIN_LAT(15),
      .CLK_COR_PRECEDENCE("TRUE"),
      .CLK_COR_REPEAT_WAIT(0),
      .CLK_COR_SEQ_LEN(1),
      .CLK_COR_SEQ_1_ENABLE(4'hf),
      .CLK_COR_SEQ_1_1(10'h100),
      .CLK_COR_SEQ_1_2(10'h100),
      .CLK_COR_SEQ_1_3(10'h000),
      .CLK_COR_SEQ_1_4(10'h000),
      .CLK_CORRECT_USE("FALSE"),
      .CLK_COR_SEQ_2_ENABLE(4'hf),
      .CLK_COR_SEQ_2_1(10'h100),
      .CLK_COR_SEQ_2_2(10'h000),
      .CLK_COR_SEQ_2_3(10'h000),
      .CLK_COR_SEQ_2_4(10'h000),

      .CHAN_BOND_KEEP_ALIGN("FALSE"),
      .CHAN_BOND_MAX_SKEW(1),
      .CHAN_BOND_SEQ_LEN(1),
      .CHAN_BOND_SEQ_1_1(10'h000),
      .CHAN_BOND_SEQ_1_2(10'h000),
      .CHAN_BOND_SEQ_1_3(10'h000),
      .CHAN_BOND_SEQ_1_4(10'h000),
      .CHAN_BOND_SEQ_1_ENABLE(4'hf),
      .CHAN_BOND_SEQ_2_1(10'h000),
      .CHAN_BOND_SEQ_2_2(10'h000),
      .CHAN_BOND_SEQ_2_3(10'h000),
      .CHAN_BOND_SEQ_2_4(10'h000),
      .CHAN_BOND_SEQ_2_ENABLE(4'hf),
      .CHAN_BOND_SEQ_2_USE("FALSE"),
      .FTS_DESKEW_SEQ_ENABLE(4'hf),
      .FTS_LANE_DESKEW_CFG(4'hf),
      .FTS_LANE_DESKEW_EN("FALSE"),

      .ES_CONTROL(6'h00),
      .ES_ERRDET_EN("FALSE"),
      .ES_EYE_SCAN_EN("TRUE"),
      .ES_HORZ_OFFSET(12'h000),
      .ES_PMA_CFG(10'h000),
      .ES_PRESCALE(5'h00),
      .ES_QUALIFIER(80'h00000000000000000000),
      .ES_QUAL_MASK(80'h00000000000000000000),
      .ES_SDATA_MASK(80'h00000000000000000000),
      .ES_VERT_OFFSET(9'h000),

      .RX_DATA_WIDTH(40),

      .OUTREFCLK_SEL_INV(2'h3),
      .PMA_RSV(32'h00018480),
      .PMA_RSV2(16'h2050),
      .PMA_RSV3(2'h0),
      .PMA_RSV4(32'h00000000),
      .RX_BIAS_CFG(12'h004),
      .DMONITOR_CFG(24'h000a00),
      .RX_CM_SEL(2'h3),
      .RX_CM_TRIM(3'h2),
      .RX_DEBUG_CFG(12'h000),
      .RX_OS_CFG(13'h0080),
      .TERM_RCAL_CFG(5'h10),
      .TERM_RCAL_OVRD(1'b0),
      .TST_RSV(32'h00000000),
      .RX_CLK25_DIV(5),
      .TX_CLK25_DIV(5),
      .UCODEER_CLR(1'b0),

      .PCS_PCIE_EN("FALSE"),

      .PCS_RSVD_ATTR(48'h0000000001c0),

      .RXBUF_ADDR_MODE("FAST"),
      .RXBUF_EIDLE_HI_CNT(4'h8),
      .RXBUF_EIDLE_LO_CNT(4'h0),
      .RXBUF_EN("FALSE"),
      .RX_BUFFER_CFG(6'h00),
      .RXBUF_RESET_ON_CB_CHANGE("TRUE"),
      .RXBUF_RESET_ON_COMMAALIGN("FALSE"),
      .RXBUF_RESET_ON_EIDLE("FALSE"),
      .RXBUF_RESET_ON_RATE_CHANGE("TRUE"),
      .RXBUFRESET_TIME(5'h01),
      .RXBUF_THRESH_OVFLW(61),
      .RXBUF_THRESH_OVRD("FALSE"),
      .RXBUF_THRESH_UNDFLW(4),
      .RXDLY_CFG(16'h001f),
      .RXDLY_LCFG(9'h030),
      .RXDLY_TAP_CFG(16'h0000),
      .RXPH_CFG(24'h000000),
      .RXPHDLY_CFG(24'h084020),
      .RXPH_MONITOR_SEL(5'h00),
      .RX_XCLK_SEL("RXUSR"),
      .RX_DDI_SEL(6'h00),
      .RX_DEFER_RESET_BUF_EN("TRUE"),

      .RXCDR_CFG(72'h0380008bff10200010),
      .RXCDR_FR_RESET_ON_EIDLE(1'b0),
      .RXCDR_HOLD_DURING_EIDLE(1'b0),
      .RXCDR_PH_RESET_ON_EIDLE(1'b0),
      .RXCDR_LOCK_CFG(6'h15),

      .RXCDRFREQRESET_TIME(5'h01),
      .RXCDRPHRESET_TIME(5'h01),
      .RXISCANRESET_TIME(5'h01),
      .RXPCSRESET_TIME(5'h01),
      .RXPMARESET_TIME(5'h03),

      .RXOOB_CFG(7'h06),

      .RXGEARBOX_EN("FALSE"),
      .GEARBOX_MODE(3'h0),

      .RXPRBS_ERR_LOOPBACK(1'b0),

      .PD_TRANS_TIME_FROM_P2(12'h03c),
      .PD_TRANS_TIME_NONE_P2(8'h3c),
      .PD_TRANS_TIME_TO_P2(8'h64),

      .SAS_MAX_COM(64),
      .SAS_MIN_COM(36),
      .SATA_BURST_SEQ_LEN(4'hf),
      .SATA_BURST_VAL(3'h4),
      .SATA_EIDLE_VAL(3'h4),
      .SATA_MAX_BURST(8),
      .SATA_MAX_INIT(21),
      .SATA_MAX_WAKE(7),
      .SATA_MIN_BURST(4),
      .SATA_MIN_INIT(12),
      .SATA_MIN_WAKE(4),

      .TRANS_TIME_RATE(8'h0e),

      .TXBUF_EN("TRUE"),
      .TXBUF_RESET_ON_RATE_CHANGE("TRUE"),
      .TXDLY_CFG(16'h001f),
      .TXDLY_LCFG(9'h030),
      .TXDLY_TAP_CFG(16'h0000),
      .TXPH_CFG(16'h0780),
      .TXPHDLY_CFG(24'h084020),
      .TXPH_MONITOR_SEL(5'h00),
      .TX_XCLK_SEL("TXOUT"),

      .TX_DATA_WIDTH(40),

      .TX_DEEMPH0(5'h00),
      .TX_DEEMPH1(5'h00),
      .TX_EIDLE_ASSERT_DELAY(3'h6),
      .TX_EIDLE_DEASSERT_DELAY(3'h4),
      .TX_LOOPBACK_DRIVE_HIZ("FALSE"),
      .TX_MAINCURSOR_SEL(1'b0),
      .TX_DRIVE_MODE("DIRECT"),
      .TX_MARGIN_FULL_0(7'h4e),
      .TX_MARGIN_FULL_1(7'h49),
      .TX_MARGIN_FULL_2(7'h45),
      .TX_MARGIN_FULL_3(7'h42),
      .TX_MARGIN_FULL_4(7'h40),
      .TX_MARGIN_LOW_0(7'h46),
      .TX_MARGIN_LOW_1(7'h44),
      .TX_MARGIN_LOW_2(7'h42),
      .TX_MARGIN_LOW_3(7'h40),
      .TX_MARGIN_LOW_4(7'h40),

      .TXGEARBOX_EN("FALSE"),

      .TXPCSRESET_TIME(5'h01),
      .TXPMARESET_TIME(5'h01),

      .TX_RXDETECT_CFG(refclk_freq),
      .TX_RXDETECT_REF(3'h4),

      .CPLL_CFG(24'hbc07dc),
      .CPLL_FBDIV(4),
      .CPLL_FBDIV_45(5),
      .CPLL_INIT_CFG(24'h00001e),
      .CPLL_LOCK_CFG(16'h01e8),
      .CPLL_REFCLK_DIV(1),
      .RXOUT_DIV(1),
      .TXOUT_DIV(1),
      .SATA_CPLL_CFG("VCO_3000MHZ"),

      .RXDFELPMRESET_TIME(7'h0f),

      .RXLPM_HF_CFG(14'h00f0),
      .RXLPM_LF_CFG(14'h00f0),
      .RX_DFE_GAIN_CFG(23'h020fea),
      .RX_DFE_H2_CFG(12'h000),
      .RX_DFE_H3_CFG(12'h040),
      .RX_DFE_H4_CFG(11'h0f0),
      .RX_DFE_H5_CFG(11'h0e0),
      .RX_DFE_KL_CFG(13'h00fe),
      .RX_DFE_LPM_CFG(16'h0954),
      .RX_DFE_LPM_HOLD_DURING_EIDLE(1'b0),
      .RX_DFE_UT_CFG(17'h11e00),
      .RX_DFE_VP_CFG(17'h03f03),

      .RX_CLKMUX_PD(1'b1),
      .TX_CLKMUX_PD(1'b1),

      .RX_INT_DATAWIDTH(1),

      .TX_INT_DATAWIDTH(1),

      .TX_QPI_STATUS_EN(1'b0),

      .RX_DFE_KL_CFG2(32'h301148ac),
      .RX_DFE_XYD_CFG(13'h0000),

      .TX_PREDRIVER_MODE(1'b0)
      )
   gtxe2_i
     (
      .CPLLFBCLKLOST(),
      .CPLLLOCK(cpll_lock),
      .CPLLLOCKDETCLK(1'b0),
      .CPLLLOCKEN(1'b1),
      .CPLLPD(!refclk_locked_sync),
      .CPLLREFCLKLOST(),
      .CPLLREFCLKSEL(3'h1),
      .CPLLRESET(cpll_reset),
      .GTRSVD(16'h0000),
      .PCSRSVDIN(16'h0000),
      .PCSRSVDIN2(5'h00),
      .PMARSVDIN(5'h00),
      .PMARSVDIN2(5'h00),
      .TSTIN(20'hfffff),
      .TSTOUT(),

      .CLKRSVD(4'd0),

      .GTGREFCLK(1'b0),
      .GTNORTHREFCLK0(1'b0),
      .GTNORTHREFCLK1(1'b0),
      .GTREFCLK0(q2_clk0_refclk_i),
      .GTREFCLK1(1'b0),
      .GTSOUTHREFCLK0(1'b0),
      .GTSOUTHREFCLK1(1'b0),

      .DRPADDR(9'd0),
      .DRPCLK(1'b0),
      .DRPDI(16'd0),
      .DRPDO(),
      .DRPEN(1'b0),
      .DRPRDY(),
      .DRPWE(1'b0),

      .GTREFCLKMONITOR(),
      .QPLLCLK(),
      .QPLLREFCLK(),
      .RXSYSCLKSEL(2'h0),
      .TXSYSCLKSEL(2'h0),

      .DMONITOROUT(),

      .TX8B10BEN(1'b1),

      .LOOPBACK(3'h0),

      .PHYSTATUS(phystatus),
      .RXRATE(3'h0),
      .RXVALID(),

      .RXPD(powerdown),
      .TXPD(powerdown),

      .SETERRSTATUS(1'b0),

      .EYESCANRESET(1'b0),
      .RXUSERRDY(1'b1),

      .EYESCANDATAERROR(),
      .EYESCANMODE(1'b0),
      .EYESCANTRIGGER(1'b0),

      .RXCDRFREQRESET(1'b0),
      .RXCDRHOLD(1'b0),
      .RXCDRLOCK(),
      .RXCDROVRDEN(1'b0),
      .RXCDRRESET(1'b0),
      .RXCDRRESETRSV(1'b0),

      .RXCLKCORCNT(),

      .RX8B10BEN(1'b1),

      .RXUSRCLK(pipe_rx_clk),
      .RXUSRCLK2(pipe_rx_clk),

      .RXDATA(pipe_rx),

      .RXPRBSERR(),
      .RXPRBSSEL(3'h0),

      .RXPRBSCNTRESET(1'b0),

      .RXDFEXYDEN(1'b1),
      .RXDFEXYDHOLD(1'b0),
      .RXDFEXYDOVRDEN(1'b0),

      .RXDISPERR(),
      .RXNOTINTABLE(),

      .GTXRXP(gtx_rxp),

      .GTXRXN(gtx_rxn),

      .RXBUFRESET(1'b0),
      .RXBUFSTATUS(),
      .RXDDIEN(1'b1),
      .RXDLYBYPASS(1'b0),
      .RXDLYEN(1'b0),
      .RXDLYOVRDEN(1'b0),
      .RXDLYSRESET(rxdlysreset),
      .RXDLYSRESETDONE(rxdlysresetdone),
      .RXPHALIGN(1'b0),
      .RXPHALIGNDONE(rxphaligndone),
      .RXPHALIGNEN(1'b0),
      .RXPHDLYPD(1'b0),
      .RXPHDLYRESET(1'b0),
      .RXPHMONITOR(),
      .RXPHOVRDEN(1'b0),
      .RXPHSLIPMONITOR(),
      .RXSTATUS(rxstatus),

      .RXBYTEISALIGNED(),
      .RXBYTEREALIGN(),
      .RXCOMMADET(),
      .RXCOMMADETEN(1'b1),
      .RXMCOMMAALIGNEN(rx_align_enable_d2),
      .RXPCOMMAALIGNEN(rx_align_enable_d2),

      .RXCHANBONDSEQ(),
      .RXCHBONDEN(1'b0),
      .RXCHBONDLEVEL(3'h0),
      .RXCHBONDMASTER(1'b0),
      .RXCHBONDO(),
      .RXCHBONDSLAVE(1'b0),

      .RXCHANISALIGNED(),
      .RXCHANREALIGN(),

      .RXLPMHFHOLD(1'b0),
      .RXLPMHFOVRDEN(1'b0),
      .RXLPMLFHOLD(1'b0),

      .RXDFEAGCHOLD(1'b0),
      .RXDFEAGCOVRDEN(1'b0),
      .RXDFECM1EN(1'b0),
      .RXDFELFHOLD(1'b0),
      .RXDFELFOVRDEN(1'b1),
      .RXDFELPMRESET(1'b0),
      .RXDFETAP2HOLD(1'b0),
      .RXDFETAP2OVRDEN(1'b0),
      .RXDFETAP3HOLD(1'b0),
      .RXDFETAP3OVRDEN(1'b0),
      .RXDFETAP4HOLD(1'b0),
      .RXDFETAP4OVRDEN(1'b0),
      .RXDFETAP5HOLD(1'b0),
      .RXDFETAP5OVRDEN(1'b0),
      .RXDFEUTHOLD(1'b0),
      .RXDFEUTOVRDEN(1'b0),
      .RXDFEVPHOLD(1'b0),
      .RXDFEVPOVRDEN(1'b0),
      .RXDFEVSEN(1'b0),
      .RXLPMLFKLOVRDEN(1'b0),
      .RXMONITOROUT(),
      .RXMONITORSEL(2'h0),
      .RXOSHOLD(1'b0),
      .RXOSOVRDEN(1'b0),

      .RXRATEDONE(),

      .RXOUTCLK(gt0_rxoutclk_i),
      .RXOUTCLKFABRIC(),
      .RXOUTCLKPCS(),
      .RXOUTCLKSEL(3'h2),

      .RXDATAVALID(),
      .RXHEADER(),
      .RXHEADERVALID(),
      .RXSTARTOFSEQ(),

      .RXGEARBOXSLIP(1'b0),

      .GTRXRESET(reset_rx),
      .RXOOBRESET(1'b0),
      .RXPCSRESET(1'b0),
      .RXPMARESET(1'b0),

      .RXLPMEN(1'b0),

      .RXCOMSASDET(),
      .RXCOMWAKEDET(),

      .RXCOMINITDET(),

      .RXELECIDLE(gt0_rxelecidle_i),
      .RXELECIDLEMODE(2'h0),

      .RXPOLARITY(rxpolarity),

      .RXSLIDE(1'b0),

      .RXCHARISCOMMA(),
      .RXCHARISK(pipe_rx_k),

      .RXCHBONDI(5'h00),

      .RXRESETDONE(rxresetdone),

      .RXQPIEN(1'b0),
      .RXQPISENN(),
      .RXQPISENP(),

      .TXPHDLYTSTCLK(1'b0),

      .TXPOSTCURSOR(5'h00),
      .TXPOSTCURSORINV(1'b0),
      .TXPRECURSOR(5'h00),
      .TXPRECURSORINV(1'b0),
      .TXQPIBIASEN(1'b0),
      .TXQPISTRONGPDOWN(1'b0),
      .TXQPIWEAKPUP(1'b0),

      .CFGRESET(1'b0),
      .GTTXRESET(reset_tx),
      .PCSRSVDOUT(),
      .TXUSERRDY(1'b1),

      .GTRESETSEL(1'b0),
      .RESETOVRD(1'b0),

      .TXCHARDISPMODE({4'd0, { 4{txdata_extend} } }),
      .TXCHARDISPVAL({4'd0, { 4{txdata_extend} } }),

      .TXUSRCLK(usrclk),
      .TXUSRCLK2(usrclk),

      .TXELECIDLE(txelecidle),
      .TXMARGIN(3'h0),
      .TXRATE(3'h0),
      .TXSWING(1'b0),

      .TXPRBSFORCEERR(1'b0),

      .TXDLYBYPASS(1'b1),
      .TXDLYEN(1'b0),
      .TXDLYHOLD(1'b0),
      .TXDLYOVRDEN(1'b0),
      .TXDLYSRESET(1'b0),
      .TXDLYSRESETDONE(),
      .TXDLYUPDOWN(1'b0),
      .TXPHALIGN(1'b0),
      .TXPHALIGNDONE(),
      .TXPHALIGNEN(1'b0),
      .TXPHDLYPD(1'b0),
      .TXPHDLYRESET(1'b0),
      .TXPHINIT(1'b0),
      .TXPHINITDONE(),
      .TXPHOVRDEN(1'b0),

      .TXBUFSTATUS(),

      .TXBUFDIFFCTRL(3'h4),
      .TXDEEMPH(1'b0),
      .TXDIFFCTRL(4'hc),
      .TXDIFFPD(1'b0),
      .TXINHIBIT(1'b0),
      .TXMAINCURSOR(7'h00),
      .TXPISOPD(1'b0),

      .TXDATA(txdata),

      .GTXTXN(gtx_txn),
      .GTXTXP(gtx_txp),

      .TXOUTCLK(gt0_txoutclk_i),
      .TXOUTCLKFABRIC(),
      .TXOUTCLKPCS(),
      .TXOUTCLKSEL(3'h3),
      .TXRATEDONE(),

      .TXCHARISK(txdata_k),
      .TXGEARBOXREADY(),
      .TXHEADER(3'h0),
      .TXSEQUENCE(7'h00),
      .TXSTARTSEQ(1'b0),

      .TXPCSRESET(1'b0),
      .TXPMARESET(1'b0),
      .TXRESETDONE(txresetdone),

      .TXCOMFINISH(),
      .TXCOMINIT(1'b0),
      .TXCOMSAS(1'b0),
      .TXCOMWAKE(1'b0),
      .TXPDELECIDLEMODE(1'b1),

      .TXPOLARITY(1'b0),

      .TXDETECTRX(txdetectrx),

      .TX8B10BBYPASS({ 4'd0, {4{tx_coding_bypass}}}),

      .TXPRBSSEL(3'h0),

      .TXQPISENN(),
      .TXQPISENP()

      );

   IBUFDS_GTE2 ibufgte
     (
      .O(q2_clk0_refclk_i),
      .ODIV2(),
      .CEB(1'b0),
      .I(gtx_refclk_p),
      .IB(gtx_refclk_n)
      );

   BUFG usrclk_bufg
     (.I(gt0_txoutclk_i), .O(usrclk));

   BUFG pipe_rx_bufg
     (.I(gt0_rxoutclk_i), .O(pipe_rx_clk));

endmodule
